@@ -17,12 +17,14 @@
 import sys
 from calculate.core.server.func import Action, Tasks
 from calculate.lib.cl_lang import setLocalTranslate, getLazyLocalTranslate
+from calculate.lib.cl_template import TemplatesError
 from calculate.lib.utils.colortext import get_color_print, Colors
 from calculate.lib.utils.files import FilesError
 from calculate.update.update import UpdateError
 from calculate.update.emerge_parser import EmergeError
 from calculate.lib.utils.portage import GitError, Eix, EmergeLog, \
     EmergeLogNamedTask, PackageList
+from ..update_tasks import EmergeMark
 
 setLocalTranslate('cl_update3', sys.modules[__name__])
 __ = getLazyLocalTranslate(_)
@@ -33,7 +35,9 @@ class ClUpdateAction(Action):
     Действие обновление конфигурационных файлов
     """
     # ошибки, которые отображаются без подробностей
-    native_error = (FilesError, UpdateError, GitError, EmergeError)
+    native_error = (FilesError, UpdateError,
+                    TemplatesError,
+                    GitError, EmergeError)
 
     successMessage = None
     failedMessage = None
@@ -53,6 +57,7 @@ class ClUpdateAction(Action):
         return func
 
     def need_upgrade(pkg):
+        # TODO: возможно функция не работает
         def func():
             return bool(Eix(pkg, Eix.Option.Upgrade).get_packages())
         return func
@@ -60,16 +65,6 @@ class ClUpdateAction(Action):
     def pkg_color(text):
         return text
 
-    log_names = {'schedule': "schedule",
-                 'premerge': "check updates",
-                 'python_updater': "update python modules",
-                 'perl_cleaner': "update perl modules",
-                 'kernel_modules': "update kernel modules",
-                 'depclean': "depclean",
-                 'xorg_modules': "update xorg modules",
-                 'preserved_libs': "update preserved libs",
-                 'revdep': "revdep rebuild"}
-
     emerge_tasks = [
         {'name': 'premerge_group',
          'group': __("Checking for updates"),
@@ -93,7 +88,7 @@ class ClUpdateAction(Action):
          'tasks': [
              {'name': 'update:update_world',
               'message': __("Calculating dependencies"),
-              'method': 'Update.emerge("-uDN","--with-bdeps=y","@world")',
+              'method': 'Update.emerge("","-uDN","--with-bdeps=y","@world")',
              }
          ]
         },
@@ -105,9 +100,9 @@ class ClUpdateAction(Action):
                             'to a Python upgrade'),
               'method': 'Update.emergelike("python-updater")',
               'condition': was_installed('dev-lang/python$',
-                                         log_names['python_updater']),
-              'decoration': 'Update.update_task("%s")' % log_names[
-                  'python_updater']
+                                         EmergeMark.PythonUpdater),
+              'decoration': 'Update.update_task("%s")' %
+                            EmergeMark.PythonUpdater
              },
          ]
         },
@@ -119,9 +114,8 @@ class ClUpdateAction(Action):
                             'broken due to a perl upgrade'),
               'method': 'Update.emergelike("perl-cleaner", "all")',
               'condition': was_installed('dev-lang/perl$',
-                                         log_names['perl_cleaner']),
-              'decoration': 'Update.update_task("%s")' % log_names[
-                  'perl_cleaner']
+                                         EmergeMark.PerlCleaner),
+              'decoration': 'Update.update_task("%s")' % EmergeMark.PerlCleaner
              },
          ]
         },
@@ -131,8 +125,8 @@ class ClUpdateAction(Action):
              {'name': 'update_other:update_depclean',
               'message': __("Calculating dependencies"),
               'method': 'Update.depclean()',
-              'condition': need_depclean('.*', log_names['depclean']),
-              'decoration': 'Update.update_task("%s")' % log_names['depclean']
+              'condition': need_depclean('.*', EmergeMark.Depclean),
+              'decoration': 'Update.update_task("%s")' % EmergeMark.Depclean
              },
          ]
         },
@@ -141,34 +135,35 @@ class ClUpdateAction(Action):
          'tasks': [
              {'name': 'update_other:module_rebuild',
               'message': __('Updating Kernel modules'),
-              'method': 'Update.emerge("@module-rebuild")',
+              'method': 'Update.emerge("","@module-rebuild")',
               'condition': was_installed('sys-kernel/.*source',
-                                         log_names['kernel_modules']),
-              'decoration': 'Update.update_task("%s")' % log_names[
-                  'kernel_modules']
+                                         EmergeMark.KernelModules),
+              'decoration': 'Update.update_task("%s")' %
+                            EmergeMark.KernelModules
              },
              {'name': 'update_other:x11_module_rebuild',
               'message': __('Updating X.Org server modules'),
-              'method': 'Update.emerge("@x11-module-rebuild")',
+              'method': 'Update.emerge("","@x11-module-rebuild")',
               'condition': was_installed('x11-base/xorg-server',
-                                         log_names['xorg_modules']),
-              'decoration': 'Update.update_task("%s")' % log_names[
-                  'xorg_modules']
+                                         EmergeMark.XorgModules),
+              'decoration': 'Update.update_task("%s")' %
+                            EmergeMark.XorgModules
              },
              {'name': 'update_other:preserved_rebuild',
               'message': __('Updating preserved libraries'),
-              'method': 'Update.emerge("@preserved-rebuild")',
-              'condition': was_installed('.*', log_names['preserved_libs']),
-              'decoration': 'Update.update_task("%s")' % log_names[
-                  'preserved_libs']
+              'method': 'Update.emerge("","@preserved-rebuild")',
+              'condition': was_installed('.*', EmergeMark.PreservedLibs),
+              'decoration': 'Update.update_task("%s")' %
+                            EmergeMark.PreservedLibs
              },
              {'name': 'update_other:revdev_rebuild',
               'message': __('Checking reverse dependencies'),
               'method': 'Update.revdep_rebuild("revdep-rebuild")',
               'condition': lambda Get:(Get('cl_update_skip_rb_set') == 'off'
-                    and ClUpdateAction.was_installed('.*', 
-                            ClUpdateAction.log_names['revdep'])()),
-              'decoration': 'Update.update_task("%s")' % log_names['revdep']
+                    and ClUpdateAction.was_installed('.*',
+                            EmergeMark.RevdepRebuild)()),
+              'decoration': 'Update.update_task("%s")' %
+                            EmergeMark.RevdepRebuild
              },
              {'name': 'update_other:dispatch_conf_end',
               'message': __("Updating configuration files"),
@@ -183,7 +178,7 @@ class ClUpdateAction(Action):
         }
     ]
 
-    # список задач для дейсвия
+    # список задач для действия
     tasks = [
         {'name': 'check_schedule',
          'method': 'Update.checkSchedule(cl_update_autocheck_interval,'
@@ -197,12 +192,32 @@ class ClUpdateAction(Action):
         {'name': 'reps_synchronization',
          'group': __("Repositories synchronization"),
          'tasks': [
+             # запасная синхронизация, в ходе которой ветки обновляются до
+             # master
+             {'name': 'sync_reps_fallback',
+              'foreach': 'cl_update_sync_rep',
+              'message':
+                  __("Fallback syncing the {eachvar:capitalize} repository"),
+              'method': 'Update.syncRepositories(eachvar)',
+              'condition': lambda Get: ("getbinpkg" in Get('cl_features') and
+                        not Get('cl_update_binhost_data')[0])
+             },
+             # обновление переменных информации из binhost
+             {'name': 'update_binhost_list',
+              'method': 'Update.update_binhost_list()',
+              'condition': lambda Get: ("getbinpkg" in Get('cl_features') and
+                                        not Get('cl_update_binhost_data')[0])
+             },
              {'name': 'sync_reps',
               'foreach': 'cl_update_sync_rep',
               'message': __("Syncing the {eachvar:capitalize} repository"),
               'method': 'Update.syncRepositories(eachvar)',
               'condition': lambda Get: Get('cl_update_sync_rep')
              },
+             {'name': 'check_binhost',
+              'method': 'Update.check_binhost()',
+              'condition': lambda Get: "getbinpkg" in Get('cl_features')
+             },
              {'name': 'sync_other_reps',
               'foreach': 'cl_update_other_rep_name',
               'message': __("Syncing the {eachvar:capitalize} repository"),
@@ -233,14 +248,14 @@ class ClUpdateAction(Action):
              },
              {'name': 'eix_update',
               'message': __("Updating the eix cache"),
-              'method': 'Update.eixUpdate()',
+              'method': 'Update.eixUpdate(cl_repository_name)',
               'condition': (
                   lambda Get: (Get('cl_update_outdate_set') == 'on' and
                                Get('cl_update_eixupdate_force') != 'skip' or
                                Get('cl_update_eixupdate_force') == 'force'))
              },
              {'name': 'update_setup_cache',
-              'message': __("Updating the package setup cache"),
+              'message': __("Updating the cache of configurable packages"),
               'method': 'Update.updateSetupCache()',
               'essential': False,
               'condition': lambda Get: Get('cl_update_outdate_set') == 'on'
@@ -277,11 +292,14 @@ class ClUpdateAction(Action):
              'condition': lambda Get: (Get('cl_dispatch_conf') != 'skip' and
                                        Get('cl_update_pretend_set') == 'off')
             },
+            {'name': 'binhost_changed',
+             'method': 'Update.message_binhost_changed()'
+            },
          ]
         }
     ] + emerge_tasks + [
-        {'name':'failed',
-         'error':__("Update failed"),
+        {'name': 'failed',
+         'error': __("Update failed"),
          'depend': (Tasks.failed() & Tasks.hasnot("interrupt") &
                     (Tasks.hasnot("check_schedule") |
                      Tasks.success_all("check_schedule")))},
